%macro direction(data=, d=, n=, v1=, v2=, v3=);


/*************************************************************/
/*                                                           */
/* Compares samples of directional data in the plane (d=2)   */
/* or in the space (d=3) by means of the von Mises distribu- */
/* tion and the Fisher distribution, respectively. The       */
/* results are based on asymptotic theory and are valid      */
/* for large values of the precisions (kappas) only.         */
/* The macro reports when the conditions fi>2 and R> ?n are  */
/* not satisfied.                                            */
/* The dataset 'data' should contain the follwing variables  */
/* for each sample:                                          */
/*         'n':        number of observations                */
/*         'v1':       1. component of the resultant vector  */
/*         'v2':       2. do.                                */
/*         'v3':       3. do. (if d=3)                       */
/*                                                           */
/* Danish versions:                                          */
/* Jrn Attermann 30-12-93,07-01-94,03-02-94                 */
/* First english version:                                    */
/* Preben Bl[sild and J|rgen Granfeldt 02-02-2001            */
/*                                                           */
/*************************************************************/


%let startcol=1; /* Position of the first column in the output         */
%let alfa=0.05;  /* Significance level for the hypothesis that the     */
                 /* precisions are equal                               */

%let stop=0;     /* Stop, if 1                                         */
%let rbet=1;     /* The condition R > ?n satisfied, if 1               */
%let h1=0;       /* The hypothesis that the precisions are equal is    */ 
                 /* is not rejected, if 1                                  */
%let fbet=1;     /* The condition fi >= 2 satisfied, if 1              */

%if &d=2 %then %let v3=;

/* Calculations in template for calculations*/

data _raekker;
  set &data(keep=&n &v1 &v2 &v3) end=last;
  %if &d=2 %then %do;
    _ri=sqrt(&v1**2+&v2**2);
  %end;
  %else %if &d=3 %then %do;
    _ri=sqrt(&v1**2+&v2**2+&v3**2);
  %end;
  _ssdi=2*(&n-_ri);
  _fi=(&n-1)*(&d-1);
  if _fi > 0 then do;
    _kapinvi=_ssdi/_fi;
    _invfi=1/_fi;
  end;
  else do;
    call symput('stop','1');
    file print;
    put / @32+&startcol 'Macro "DIRECTION"';
    put / @32+&startcol "Dataset: %upcase(&data)";
    put / @21+&startcol 'ERROR!  Number of observations less than 2.' /;
    stop;
  end;
  if _fi < 2 then call symput('fbet','0');
  _fikapin=_fi*log(_kapinvi);
  if last then do;
    call symput('k',_n_);
    if _n_ < 2 then do;
      call symput('stop','1');
      file print;
      put / @32+&startcol 'Macro "DIRECTION"';
      put / @32+&startcol "Dataset: %upcase(&data)";
      put / @26+&startcol 'ERROR!  1 sample only.' /;
      put;
      stop;
    end;
  end;
run;

%if &stop=0 %then %do;

  proc means data=_raekker noprint;
    var &n &v1 &v2 &v3 _ri _ssdi _fi _invfi _fikapin;
    output out=_sum sum=&n &v1 &v2 &v3 _ri _ssdi _fi _invfi _fikapin;
  run;

  data _sum;
    set _sum;
    %if &d=2 %then %do;
      _r=sqrt(&v1**2+&v2**2);
      if _r <= 0.95*&n then call symput('rbet','0');
    %end;
    %else %if &d=3 %then %do;
      _r=sqrt(&v1**2+&v2**2+&v3**2);
      if _r <= 2/3*&n then call symput('rbet','0');
    %end;
    _kappain=_ssdi/_fi;
  run;


  /* Test for equality of precisions*/

  %if &k=2 %then %do;

    data _h1;
      set _raekker;
      retain _pushkap _pushfi;
      if _n_=1 then do;
        _pushkap=_kapinvi;
        _pushfi=_fi;
      end;
      else if _n_=2 then do;
        if _pushkap > _kapinvi then do;
          _kapnum=_pushkap;
          _kapden=_kapinvi;
          _finum=_pushfi;
          _fiden=_fi;
        end;
        else do;
          _kapnum=_kapinvi;
          _kapden=_pushkap;
          _finum=_fi;
          _fiden=_pushfi;
        end;
        _ftest=_kapnum/_kapden;
        _testss=2*(1-probf(_ftest,_finum,_fiden));
        if _testss > &alfa then call symput('h1','1');
        output;
      end;
      keep _finum _fiden _ftest _testss;
    run;

  %end;
  %else %do;

    data _h1;
      set _sum;
      _c=1+1/3/(&k-1)*(_invfi-1/_fi);
      _ba=1/_c*(_fi*log(_kappain)-_fikapin);
      _testss=1-probchi(_ba,&k-1);
      if _testss > &alfa then call symput('h1','1');
      keep _c _ba _testss;
    run;

  %end;


  /* Test for equality of directions*/

  data _h2;
    set _sum;
    _f2=(&k-1)*(&d-1);
    _ftest=2*(_ri-_r)/_f2/_kappain;
    _testss=1-probf(_ftest,_f2,_fi);
    keep _r _ri _ftest _f2 _fi _testss;
  run;


  /* Print calculated values*/



  data _null_;
    set _raekker _sum _h1 _h2;
    file print;
    if _n_=1 then do;
      put / @32+&startcol 'Macro "DIRECTION"';
      put / @32+&startcol "Dataset: %upcase(&data)";
      put // @27+&startcol "Calculations in k samples:";
      %if &d=2 %then %do;
        put / @1+&startcol 'i' @6+&startcol 'n' @13+&startcol 'v1' @23+&startcol 'v2'
                @36+&startcol 'R' @46+&startcol '2(n-R)' @58+&startcol 'f'
                @62+&startcol 'kappa inverse';
        put @&startcol
            '---------------------------------------------------------------------------';
      %end;
      %else %if &d=3 %then %do;
        put / @1+&startcol 'i' @6+&startcol 'n' @11+&startcol 'v1' @20+&startcol 'v2'
              @29+&startcol 'v3' @41+&startcol 'R' @50+&startcol '2(n-R)' @59+&startcol 'f'
              @62+&startcol 'kappa inverse';
        put @&startcol
            '---------------------------------------------------------------------------';
      %end;
    end;
    if _n_ <= &k then do;
      %if &d=2 %then %do;
        put  @&startcol _n_ 2.0 @4+&startcol &n 3.0
                               @10+&startcol &v1 8.4 @20+&startcol &v2 8.4
                               @30+&startcol _ri 10.4 @42+&startcol _ssdi 10.4
                               @54+&startcol _fi 6.0  @62+&startcol _kapinvi 10.4;
      %end;
      %else %if &d=3 %then %do;
        put  @&startcol _n_ 2.0 @4+&startcol &n 3.0 @8+&startcol &v1 8.4 @17+&startcol &v2 8.4
            @26+&startcol &v3 8.4 @35+&startcol _ri 10.4 @46+&startcol _ssdi 10.4
            @57+&startcol _fi 4.0 @62+&startcol _kapinvi 10.4;
      %end;
    end;
    if _n_=&k then do;
      %if &d=2 %then %do;
        put @&startcol
            '--------------------------------------------------------------=============';
      %end;
      %else %if &d=3 %then %do;
        put @&startcol
            '--------------------------------------------------------------=============';
      %end;
    end;
    if _n_=&k+1 then do;
      %if &d=2 %then %do;
        put @4+&startcol &n 3.0  @10+&startcol &v1 8.4 @20+&startcol &v2 8.4
            @30+&startcol _ri 10.4 @42+&startcol _ssdi 10.4
            @54+&startcol _fi 6.0  @62+&startcol _kappain 10.4;
      %end;
      %else %if &d=3 %then %do;
        put @4+&startcol &n 3.0  @8+&startcol &v1 8.4 @17+&startcol &v2 8.4
            @26+&startcol &v3 8.4
            @35+&startcol _ri 10.4 @46+&startcol _ssdi 10.4
            @57+&startcol _fi 4.0  @62+&startcol _kappain 10.4;
      %end;
    end;
    if _n_=&k+2 then do;
      %if &k=2 %then %do;
        put /// @24+&startcol 'F-test for equality of precisions:';
        put / @16+&startcol 'Degrees of freedom' @39+&startcol 'F-statistic'
              @54+&startcol 'p value';
        put @15+&startcol _finum 4.0 @23+&startcol _fiden 4.0
            @40+&startcol _ftest 8.4 @53+&startcol _testss 8.4;
      %end;
      %else %do;
        put /// @24+&startcol 'Bartletts test for equality of precisions:' ;
        %if &fbet=0 %then %do;
          put / @15+&startcol 'NOTE! The condition that the degrees of freedom >= 2 is not satisfied.';
        %end;
        put / @25+&startcol 'C' @35+&startcol 'Bartlett statistic'
              @58+&startcol 'p value';
        put @18+&startcol _c 10.4 @37+&startcol _ba 10.4
            @55+&startcol _testss 10.4;
      %end;
    end;
    if _n_=&k+3 then do;
      put /// @24+&startcol 'F-test for equality of mean directions:' /;
      %if &h1=0 %then %do;
        put  @15+&startcol 'NOTE! The hypothesis that the precisions are equal is rejected.' /;
      %end;
      %if &d=2 & &rbet=0 %then %do;
        put @15+&startcol 'NOTE! The condition that R > 0.95*n is not satisfied.' /;
      %end;
      %else %if &d=3 & &rbet=0 %then %do;
        put @15+&startcol 'NOTE! The condition that R > 2/3*n is not satisfied.' /;
      %end;
      put  @18+&startcol 'R.' @32+&startcol 'R'
           @46+&startcol 'F-statistic' @59+&startcol 'p value';
      put @13+&startcol _ri 10.4 @27+&startcol _r 10.4
          @45+&startcol _ftest 10.4 @56+&startcol _testss 10.4;
    end;
  run;




%end;


%mend direction;
