%macro tgraph(data=_last_, var= , bygroup= ,df=, table=no, graphics=yes, replay=no);

/* This macro produces fractile diagrams and corresponding tables
   (optional) for assesing whether or not the values of the 
   variable "var" for every level of the classification variable 
   "bygroup" may be considered as a sample from the t distribution
   with df degrees of freedom.

   Reference:
      Preben Blsild og Jrgen Granfeldt:
      STATISTICS FOR BIOLOGISTS OG GEOLOGISTS, Section 2.4;

   For each value of the classification variable a fractile diagram
   and a corresponding table with calculations is produced and finally
   the different fractiles diagrams are plotted in the same figure.

   The same scale on the data axis is used in all the diagrams.
   The appearence of the data axis in the diagrams may determined
   by a "AXIS2" definition in the main program before the macro is
   applied. (If label=(h=3 pct) value=(h=2.7 pct) is used in the 
   definition of "AXIS2" the units on the data axis are similar to those
   on the second axis.)

   The option table=yes (default) implies that the macro prints tables 
   with calculations of the points in the fractiles diagrams. If the 
   value of "table" is different from "yes" no tables are produced.

   The option "graphics" determines the resolution of the fractile diagrams.
   If grahpics=yes (default) the plots are produced in high resolution
   graphics. If grahpics=no the figures appear in the OUTPUT window ( and 
   no common plot for the different samples is shown).

   The option replay=yes allows the plots to be replayed using PROC REPLAY.
   By default the value of "replay" is "no".                               */




axis1 label=(h=3 pct angle=90)
      value=(h=2.7 pct)
      width=2.0
      order=(-4 to 4 by 1)
      ;

data _mact1;                      /* the variable "_none" is           */
  set &data;                      /* used if "bygroup" is not          */
  _none=1;                        /* defined                           */
  IF &var NE .;                   /* It is important to exclude        */
                                  /* missing values of the variable;   */
                                  /* for SAS treats a missing value as */
                                  /* smaller than any other value, and */
                                  /* cumulated frequencies become      */
                                  /* wrong. Hence the                  */
                                  /* IF &var NE .;                     */
                                  /* statement.                        */

%if &bygroup= %then %let bygroup=_none;

proc sort data=_mact1;            /* The dataset is sorted after */
  by &bygroup;                    /* "bygroup" for use in the  */
                                  /* procedure FREQ              */

proc freq data=_mact1;            /* PROC FREQ produces the data-*/
  table &var /noprint out=_mact2; /* set _mact2 containing the   */
  by &bygroup;                    /* frequency of every value  */
run;                              /* of "var"                    */

data _mact1;                      /* Produces the dataset        */
  set _mact2;                     /* _mact1 containing the first */
  by &bygroup;                    /* three columns in table 2.2*/
  retain _cumct _pushcum;         /* in Chapter 2.               */
  if first.&bygroup then do;
    _cumct=0;
    _pushcum=0;
  end;
  _cumct+count;
  output;
  _pushcum=_cumct;

data _mact2(keep=&bygroup _nobs);   /* Produces the dataset        */
  set _mact1;                       /* _mact2 containing the num-  */
  by &bygroup;                      /* ber of observations for     */
  if last.&bygroup then             /* each level of "bygroup"     */
                   _nobs=_cumct;
  else delete;

data _mact3;                      /* Produces the dataset        */
  merge _mact1 _mact2;            /* _mact3 containing the       */
  by &bygroup;                    /* information in _mact1       */
                                  /* and _mact2                  */

data _mact1;                      /* Calculate p og p-fractile   */
  set _mact3;                     /* of the t-distribution  */
  _p=(_cumct+_pushcum)/(_nobs*2);
  _tfractile=tinv(_p,&df);
  label count='frequency' _cumct='cumulated frequency'
        _p='p' _tfractile="p fractile in t(&df)-distribution";
  format _p 5.3 _tfractile 5.3;

data _graplot;                    /* data for PROC GPLOT        */
  set _mact1;
  format _tfractile 5.0;

%if &bygroup=_none %then %let bygroup=;

%if &table=yes %then %do;
  proc print data=_mact1          /* Print table               */
             label noobs;
    var &var count
        _cumct _p _tfractile;
    by &bygroup;
    title5 'table';
%end;

title5;

%if &graphics=no %then %do;
  proc plot data=_mact1 nolegend;
    plot _tfractile*&var;
    by &bygroup;
%end;
%else %if &replay=no %then %do;
  symbol v=dot h=2 pct i=none;
  proc gplot data=_graplot ;
    plot _tfractile*&var /
    vaxis=axis1 haxis=axis2
    frame;
    by &bygroup;
    %if &bygroup= %then %do;%end;
    %else %do;
      proc gplot data=_graplot ;
        plot _tfractile*&var = &bygroup/
        vaxis=axis1 haxis=axis2
        frame;
   %end;
%end;
%else %if &replay=yes %then %do;
  symbol v=dot  h=2 pct i=none;
  proc gplot data=_graplot GOUT=normplot ;
    plot _tfractile*&var /
    vaxis=axis1 haxis=axis2
    frame;
    by &bygroup;
    %if &bygroup= %then %do; %end;
    %else %do;
     proc gplot data=_graplot GOUT=normplot;
       plot _tfractile*&var = &bygroup/
       vaxis=axis1 haxis=axis2
       frame;
   %end;
%end;

run;
quit;

proc datasets nolist;              /* Clean up! */
  delete _mact1 _mact2 _mact3 _graplot ;

run;
quit;

title5;                            /* Clean up! */

%mend tgraph;


